
package com.tmerclub.cloud.common.order.bo;

import com.tmerclub.cloud.common.order.vo.OrderItemLangVO;
import io.swagger.v3.oas.annotations.media.Schema;

import java.io.Serial;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * @author tmerclub
 * @date 2021/2/5
 */
public class EsOrderItemBO implements Serializable {
    @Serial
    private static final long serialVersionUID = 1L;
    @Schema(description = "商品图片", requiredMode = Schema.RequiredMode.REQUIRED)
    private String pic;

    @Schema(description = "中文商品名称", requiredMode = Schema.RequiredMode.REQUIRED)
    private String spuName;

    @Schema(description = "商品数量", requiredMode = Schema.RequiredMode.REQUIRED)
    private Integer count;

    @Schema(description = "商品价格", requiredMode = Schema.RequiredMode.REQUIRED)
    private Long price;

    @Schema(description = "产品购买花费积分", requiredMode = Schema.RequiredMode.REQUIRED)
    private Long useScore;

    @Schema(description = "skuId", requiredMode = Schema.RequiredMode.REQUIRED)
    private Long skuId;

    @Schema(description = "中文skuName", requiredMode = Schema.RequiredMode.REQUIRED)
    private String skuName;

    @Schema(description = "订单项id", requiredMode = Schema.RequiredMode.REQUIRED)
    private Long orderItemId;

    @Schema(description = "商品id", requiredMode = Schema.RequiredMode.REQUIRED)
    private Long spuId;

    @Schema(description = "退款状态 1.买家申请 2.卖家接受 3.买家发货 4.卖家收货 5.退款成功  -1.退款关闭")
    private Integer returnMoneySts;

    /**
     * 订单id
     */
    private Long orderId;

    /**
     * 积分价格（单价）
     */
    private Long scoreFee;

    /**
     * 中文商品名称
     */
    private String spuNameZh;

    /**
     * 英文商品名称
     */
    private String spuNameEn;

    /**
     * 中文sku名称
     */
    private String skuNameZh;

    /**
     * 英文sku名称
     */
    private String skuNameEn;

    /**
     * 中文供应商商品名称
     */
    private String supplierSpuNameZh;

    /**
     * 英文供应商商品名称
     */
    private String supplierSpuNameEn;

    /**
     * 中文供应商sku名称
     */
    private String supplierSkuNameZh;

    /**
     * 英文供应商sku名称
     */
    private String supplierSkuNameEn;

    /**
     * 供应商图片
     */
    private String supplierPic;

    /**
     * 供应商商品名称
     */
    private String supplierSpuName;

    /**
     * 供应商sku名称
     */
    private String supplierSkuName;

    /**
     * 店铺id
     */
    private Long shopId;

    /**
     * 用户Id
     */
    private Long userId;

    /**
     * 最终的退款id
     */
    private Long finalRefundId;

    /**
     * 推广员id
     */
    private Long distributionUserId;

    /**
     * 是否以评价(0.未评价1.已评价)
     */
    private Integer isComm;

    /**
     * 评论时间
     */
    private Date commTime;

    /**
     * 订单项退款状态（1:申请退款 2:退款成功 3:部分退款成功 4:退款失败）
     */
    private Integer refundStatus;

    /**
     * 0全部发货 其他数量为剩余待发货数量
     */
    private Integer beDeliveredNum;

    /**
     * 单个orderItem的配送类型 1:快递 2:自提 3：无需快递 4:同城配送
     */
    private Integer deliveryType;

    /**
     * 加入购物车时间
     */
    private Date shopCartTime;

    /**
     * 商品总金额
     */
    private Long spuTotalAmount;

    /**
     * 商品实际金额 = 商品总金额 - 分摊的优惠金额
     */
    private Long actualTotal;

    /**
     * 分摊的优惠金额
     */
    private Long shareReduce;

    /**
     * 平台优惠金额
     */
    private Long platformShareReduce;

    /**
     * 推广员佣金
     */
    private Long distributionAmount;

    /**
     * 上级推广员佣金
     */
    private Long distributionParentAmount;

    /**
     * 获得积分
     */
    private Long gainScore;

    /**
     * 店铺改价优惠金额
     */
    private Long shopChangeFreeAmount;

    /**
     * 运费减免金额
     */
    private Long freeFreightAmount;

    /**
     * 赠品金额
     */
    private Long giveawayAmount;


    /**
     * 活动id
     */
    private Long activityId;

    /**
     *  活动类型 枚举类：OrderActivityType
     */
    private Integer activityType;

    /**
     * 平台供应商佣金
     */
    private Long purchasePlatformCommission;


    /**
     * 商品编码
     */
    private String spuCode;
    /**
     * sku编码
     */
    private String partyCode;

    /**
     * 平台一级分类id
     */
    private Long primaryCategoryId;

    /**
     * 订单项国际化信息
     * @return
     */
    private List<OrderItemLangVO> orderItemLangList;

    /**
     * 组合名称
     */
    private String comboName;

    /**
     * 核销次数 -1.多次核销 0.无需核销 1.单次核销
     */
    private Integer writeOffNum;

    /**
     * 多次核销次数 -1.无限次
     */
    private Integer writeOffMultipleCount;

    /**
     * 核销开始时间
     */
    private Date writeOffStart;

    /**
     * 核销结束时间
     */
    private Date writeOffEnd;

    /**
     * 核销结束时间
     */
    private Integer spuMold;

    public Integer getSpuMold() {
        return spuMold;
    }

    public void setSpuMold(Integer spuMold) {
        this.spuMold = spuMold;
    }

    public String getComboName() {
        return comboName;
    }

    public void setComboName(String comboName) {
        this.comboName = comboName;
    }

    public Integer getWriteOffNum() {
        return writeOffNum;
    }

    public void setWriteOffNum(Integer writeOffNum) {
        this.writeOffNum = writeOffNum;
    }

    public Integer getWriteOffMultipleCount() {
        return writeOffMultipleCount;
    }

    public void setWriteOffMultipleCount(Integer writeOffMultipleCount) {
        this.writeOffMultipleCount = writeOffMultipleCount;
    }

    public Date getWriteOffStart() {
        return writeOffStart;
    }

    public void setWriteOffStart(Date writeOffStart) {
        this.writeOffStart = writeOffStart;
    }

    public Date getWriteOffEnd() {
        return writeOffEnd;
    }

    public void setWriteOffEnd(Date writeOffEnd) {
        this.writeOffEnd = writeOffEnd;
    }

    public String getPic() {
        return pic;
    }

    public void setPic(String pic) {
        this.pic = pic;
    }

    public String getSpuName() {
        return spuName;
    }

    public void setSpuName(String spuName) {
        this.spuName = spuName;
    }

    public Integer getCount() {
        return count;
    }

    public void setCount(Integer count) {
        this.count = count;
    }

    public Long getPrice() {
        return price;
    }

    public void setPrice(Long price) {
        this.price = price;
    }

    public Long getUseScore() {
        return useScore;
    }

    public void setUseScore(Long useScore) {
        this.useScore = useScore;
    }

    public Long getSkuId() {
        return skuId;
    }

    public void setSkuId(Long skuId) {
        this.skuId = skuId;
    }

    public String getSkuName() {
        return skuName;
    }

    public void setSkuName(String skuName) {
        this.skuName = skuName;
    }

    public Long getOrderItemId() {
        return orderItemId;
    }

    public void setOrderItemId(Long orderItemId) {
        this.orderItemId = orderItemId;
    }

    public Long getSpuId() {
        return spuId;
    }

    public void setSpuId(Long spuId) {
        this.spuId = spuId;
    }

    public Integer getReturnMoneySts() {
        return returnMoneySts;
    }

    public void setReturnMoneySts(Integer returnMoneySts) {
        this.returnMoneySts = returnMoneySts;
    }

    public Long getOrderId() {
        return orderId;
    }

    public void setOrderId(Long orderId) {
        this.orderId = orderId;
    }

    public Long getScoreFee() {
        return scoreFee;
    }

    public void setScoreFee(Long scoreFee) {
        this.scoreFee = scoreFee;
    }

    public String getSpuNameZh() {
        return spuNameZh;
    }

    public void setSpuNameZh(String spuNameZh) {
        this.spuNameZh = spuNameZh;
    }

    public String getSpuNameEn() {
        return spuNameEn;
    }

    public void setSpuNameEn(String spuNameEn) {
        this.spuNameEn = spuNameEn;
    }

    public String getSkuNameZh() {
        return skuNameZh;
    }

    public void setSkuNameZh(String skuNameZh) {
        this.skuNameZh = skuNameZh;
    }

    public String getSkuNameEn() {
        return skuNameEn;
    }

    public void setSkuNameEn(String skuNameEn) {
        this.skuNameEn = skuNameEn;
    }

    public String getSupplierSpuNameZh() {
        return supplierSpuNameZh;
    }

    public void setSupplierSpuNameZh(String supplierSpuNameZh) {
        this.supplierSpuNameZh = supplierSpuNameZh;
    }

    public String getSupplierSpuNameEn() {
        return supplierSpuNameEn;
    }

    public void setSupplierSpuNameEn(String supplierSpuNameEn) {
        this.supplierSpuNameEn = supplierSpuNameEn;
    }

    public String getSupplierSkuNameZh() {
        return supplierSkuNameZh;
    }

    public void setSupplierSkuNameZh(String supplierSkuNameZh) {
        this.supplierSkuNameZh = supplierSkuNameZh;
    }

    public String getSupplierSkuNameEn() {
        return supplierSkuNameEn;
    }

    public void setSupplierSkuNameEn(String supplierSkuNameEn) {
        this.supplierSkuNameEn = supplierSkuNameEn;
    }

    public String getSupplierPic() {
        return supplierPic;
    }

    public void setSupplierPic(String supplierPic) {
        this.supplierPic = supplierPic;
    }

    public String getSupplierSpuName() {
        return supplierSpuName;
    }

    public void setSupplierSpuName(String supplierSpuName) {
        this.supplierSpuName = supplierSpuName;
    }

    public String getSupplierSkuName() {
        return supplierSkuName;
    }

    public void setSupplierSkuName(String supplierSkuName) {
        this.supplierSkuName = supplierSkuName;
    }

    public Long getShopId() {
        return shopId;
    }

    public void setShopId(Long shopId) {
        this.shopId = shopId;
    }

    public Long getUserId() {
        return userId;
    }

    public void setUserId(Long userId) {
        this.userId = userId;
    }

    public Long getFinalRefundId() {
        return finalRefundId;
    }

    public void setFinalRefundId(Long finalRefundId) {
        this.finalRefundId = finalRefundId;
    }

    public Long getDistributionUserId() {
        return distributionUserId;
    }

    public void setDistributionUserId(Long distributionUserId) {
        this.distributionUserId = distributionUserId;
    }

    public Integer getIsComm() {
        return isComm;
    }

    public void setIsComm(Integer isComm) {
        this.isComm = isComm;
    }

    public Date getCommTime() {
        return commTime;
    }

    public void setCommTime(Date commTime) {
        this.commTime = commTime;
    }

    public Integer getRefundStatus() {
        return refundStatus;
    }

    public void setRefundStatus(Integer refundStatus) {
        this.refundStatus = refundStatus;
    }

    public Integer getBeDeliveredNum() {
        return beDeliveredNum;
    }

    public void setBeDeliveredNum(Integer beDeliveredNum) {
        this.beDeliveredNum = beDeliveredNum;
    }

    public Integer getDeliveryType() {
        return deliveryType;
    }

    public void setDeliveryType(Integer deliveryType) {
        this.deliveryType = deliveryType;
    }

    public Date getShopCartTime() {
        return shopCartTime;
    }

    public void setShopCartTime(Date shopCartTime) {
        this.shopCartTime = shopCartTime;
    }

    public Long getSpuTotalAmount() {
        return spuTotalAmount;
    }

    public void setSpuTotalAmount(Long spuTotalAmount) {
        this.spuTotalAmount = spuTotalAmount;
    }

    public Long getActualTotal() {
        return actualTotal;
    }

    public void setActualTotal(Long actualTotal) {
        this.actualTotal = actualTotal;
    }

    public Long getShareReduce() {
        return shareReduce;
    }

    public void setShareReduce(Long shareReduce) {
        this.shareReduce = shareReduce;
    }

    public Long getPlatformShareReduce() {
        return platformShareReduce;
    }

    public void setPlatformShareReduce(Long platformShareReduce) {
        this.platformShareReduce = platformShareReduce;
    }

    public Long getDistributionAmount() {
        return distributionAmount;
    }

    public void setDistributionAmount(Long distributionAmount) {
        this.distributionAmount = distributionAmount;
    }

    public Long getDistributionParentAmount() {
        return distributionParentAmount;
    }

    public void setDistributionParentAmount(Long distributionParentAmount) {
        this.distributionParentAmount = distributionParentAmount;
    }

    public Long getGainScore() {
        return gainScore;
    }

    public void setGainScore(Long gainScore) {
        this.gainScore = gainScore;
    }

    public Long getShopChangeFreeAmount() {
        return shopChangeFreeAmount;
    }

    public void setShopChangeFreeAmount(Long shopChangeFreeAmount) {
        this.shopChangeFreeAmount = shopChangeFreeAmount;
    }

    public Long getFreeFreightAmount() {
        return freeFreightAmount;
    }

    public void setFreeFreightAmount(Long freeFreightAmount) {
        this.freeFreightAmount = freeFreightAmount;
    }

    public Long getGiveawayAmount() {
        return giveawayAmount;
    }

    public void setGiveawayAmount(Long giveawayAmount) {
        this.giveawayAmount = giveawayAmount;
    }

    public Long getActivityId() {
        return activityId;
    }

    public void setActivityId(Long activityId) {
        this.activityId = activityId;
    }

    public Integer getActivityType() {
        return activityType;
    }

    public void setActivityType(Integer activityType) {
        this.activityType = activityType;
    }

    public Long getPurchasePlatformCommission() {
        return purchasePlatformCommission;
    }

    public void setPurchasePlatformCommission(Long purchasePlatformCommission) {
        this.purchasePlatformCommission = purchasePlatformCommission;
    }

    public String getSpuCode() {
        return spuCode;
    }

    public void setSpuCode(String spuCode) {
        this.spuCode = spuCode;
    }

    public String getPartyCode() {
        return partyCode;
    }

    public void setPartyCode(String partyCode) {
        this.partyCode = partyCode;
    }

    public List<OrderItemLangVO> getOrderItemLangList() {
        return orderItemLangList;
    }

    public void setOrderItemLangList(List<OrderItemLangVO> orderItemLangList) {
        this.orderItemLangList = orderItemLangList;
    }

    public Long getPrimaryCategoryId() {
        return primaryCategoryId;
    }

    public void setPrimaryCategoryId(Long primaryCategoryId) {
        this.primaryCategoryId = primaryCategoryId;
    }

    @Override
    public String toString() {
        return "EsOrderItemBO{" +
                "pic='" + pic + '\'' +
                ", spuName='" + spuName + '\'' +
                ", count=" + count +
                ", price=" + price +
                ", useScore=" + useScore +
                ", skuId=" + skuId +
                ", skuName='" + skuName + '\'' +
                ", orderItemId=" + orderItemId +
                ", spuId=" + spuId +
                ", returnMoneySts=" + returnMoneySts +
                ", orderId=" + orderId +
                ", scoreFee=" + scoreFee +
                ", spuNameZh='" + spuNameZh + '\'' +
                ", spuNameEn='" + spuNameEn + '\'' +
                ", skuNameZh='" + skuNameZh + '\'' +
                ", skuNameEn='" + skuNameEn + '\'' +
                ", supplierSpuNameZh='" + supplierSpuNameZh + '\'' +
                ", supplierSpuNameEn='" + supplierSpuNameEn + '\'' +
                ", supplierSkuNameZh='" + supplierSkuNameZh + '\'' +
                ", supplierSkuNameEn='" + supplierSkuNameEn + '\'' +
                ", supplierPic='" + supplierPic + '\'' +
                ", supplierSpuName='" + supplierSpuName + '\'' +
                ", supplierSkuName='" + supplierSkuName + '\'' +
                ", shopId=" + shopId +
                ", userId=" + userId +
                ", finalRefundId=" + finalRefundId +
                ", distributionUserId=" + distributionUserId +
                ", isComm=" + isComm +
                ", commTime=" + commTime +
                ", refundStatus=" + refundStatus +
                ", beDeliveredNum=" + beDeliveredNum +
                ", deliveryType=" + deliveryType +
                ", shopCartTime=" + shopCartTime +
                ", spuTotalAmount=" + spuTotalAmount +
                ", actualTotal=" + actualTotal +
                ", shareReduce=" + shareReduce +
                ", platformShareReduce=" + platformShareReduce +
                ", distributionAmount=" + distributionAmount +
                ", distributionParentAmount=" + distributionParentAmount +
                ", gainScore=" + gainScore +
                ", shopChangeFreeAmount=" + shopChangeFreeAmount +
                ", freeFreightAmount=" + freeFreightAmount +
                ", giveawayAmount=" + giveawayAmount +
                ", activityId=" + activityId +
                ", activityType=" + activityType +
                ", purchasePlatformCommission=" + purchasePlatformCommission +
                ", spuCode='" + spuCode + '\'' +
                ", partyCode='" + partyCode + '\'' +
                ", primaryCategoryId=" + primaryCategoryId +
                ", orderItemLangList=" + orderItemLangList +
                ", comboName='" + comboName + '\'' +
                ", writeOffNum=" + writeOffNum +
                ", writeOffMultipleCount=" + writeOffMultipleCount +
                ", writeOffStart=" + writeOffStart +
                ", writeOffEnd=" + writeOffEnd +
                ", spuMold=" + spuMold +
                '}';
    }
}
